'use strict';

var lezerGlsl = require('lezer-glsl');
var language = require('@codemirror/language');

/**
A language provider based on the [Lezer GLSL
parser](https://github.com/schimd-daniel/lezer-glsl), extended with
highlighting and indentation information.
*/
const glslLanguage = language.LRLanguage.define({
    name: "glsl",
    parser: lezerGlsl.parser.configure({
        props: [
            language.indentNodeProp.add({
                IfStatement: language.continuedIndent({ except: /^\s*({|else\b)/ }),
                CaseStatement: (context) => context.baseIndent + context.unit,
                BlockComment: () => null,
                CompoundStatement: language.delimitedIndent({ closing: "}" }),
                Statement: language.continuedIndent({ except: /^{/ }),
            }),
            language.foldNodeProp.add({
                "StructDeclarationList CompoundStatement": language.foldInside,
                BlockComment(tree) {
                    return { from: tree.from + 2, to: tree.to - 2 };
                },
            }),
        ],
    }),
    languageData: {
        commentTokens: { line: "//", block: { open: "/*", close: "*/" } },
        indentOnInput: /^\s*(?:case |default:|\{|\})$/,
        closeBrackets: {
            stringPrefixes: ["L", "u", "U", "u8", "LR", "UR", "uR", "u8R", "R"],
        },
    },
});
/**
Language support for GLSL.
*/
function glsl() {
    return new language.LanguageSupport(glslLanguage);
}

exports.glsl = glsl;
exports.glslLanguage = glslLanguage;
